use 5.14.0;
use strict;
use warnings;
use CairoX::Sweet::Standard;

our $VERSION = '0.0108'; # VERSION
# PODNAME: CairoX::Sweet::Core::Point

class CairoX::Sweet::Core::Point using Moose {

    has x => (
        is => 'rw',
        isa => Num,
        required => 1,
    );
    has y => (
        is => 'rw',
        isa => Num,
        required => 1,
    );

    method out {
        return ($self->x, $self->y);
    }
    method move(:$x = 0, :$y = 0) {
        $self->x($self->x + $x);
        $self->y($self->y + $y);
    }
}

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

CairoX::Sweet::Core::Point

=head1 VERSION

Version 0.0108, released 2015-03-21.

=head1 SOURCE

L<https://github.com/Csson/p5-CairoX-Sweet>

=head1 HOMEPAGE

L<https://metacpan.org/release/CairoX-Sweet>

=head1 AUTHOR

Erik Carlsson <info@code301.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2015 by Erik Carlsson <info@code301.com>.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
